/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;
import static org.fortiss.tooling.kernel.utils.KernelModelElementUtils.computeFullyName;
import static org.fortiss.tooling.kernel.utils.KernelModelElementUtils.computeRelativeName;
import static org.fortiss.tooling.kernel.utils.KernelModelElementUtils.getParentElement;

import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.fortiss.tooling.base.ui.annotation.valueprovider.EStructuralFeatureDescriptor;
import org.fortiss.tooling.kernel.model.INamedElement;
import org.fortiss.tooling.kernel.model.IProjectRootElement;

/**
 * Translates between a set of labels shown and the corresponding actual values (e.g., required for
 * {@link EReference}s.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class ComboBoxLabelValueMapping {

	/** Map: label -> actual value */
	private Map<String, Object> labelToValueMap;

	/** {@link String}-based input choice */
	Collection<String> stringLabels;

	/**
	 * Constructs a {@link ComboBoxLabelValueMapping}. {@code eStructuralFeatureDescriptor} may be
	 * {@code null} in which case this translator acts as the identity-function.
	 */
	public ComboBoxLabelValueMapping(EStructuralFeatureDescriptor eStructuralFeatureDescriptor,
			EObject modelElement, Collection<String> stringLabels) {

		if(eStructuralFeatureDescriptor == null) {
			this.stringLabels = stringLabels;
			return;
		}

		EClassifier eType = eStructuralFeatureDescriptor.getEType();
		if(eType instanceof EEnum) {
			labelToValueMap = new TreeMap<String, Object>();
			Set<String> enumValues = new TreeSet<String>();
			for(EEnumLiteral e : ((EEnum)eType).getELiterals()) {
				enumValues.add(e.getName());
				labelToValueMap.put(e.getName(), e.getInstance());
			}

		} else if(eType instanceof EClass) {

			EObject root = null;
			switch(eStructuralFeatureDescriptor.getEReferenceScope()) {
				case PROJECT:
					root = getParentElement(modelElement, IProjectRootElement.class);
					if(root != null) {
						// TODO: Other way to obtain FileProject (type is not available in this
						// plugin)
						root = root.eContainer();
					}
					break;
				case MODEL:
					root = getParentElement(modelElement, IProjectRootElement.class);
					break;
				case SUB_MODEL:
					root = modelElement;
					break;
				default:
					return;
			}

			EList<?> values = getChildrenWithType(root, eType.getInstanceClass());

			labelToValueMap = new TreeMap<String, Object>();
			for(Object v : values) {
				if(v instanceof INamedElement) {
					if(root instanceof INamedElement) {
						String label = computeRelativeName((INamedElement)root, (INamedElement)v);
						if(eStructuralFeatureDescriptor.getEReferenceScope().equals(
								EStructuralFeatureDescriptor.EReferenceScope.PROJECT)) {
							// Strip explicit qualification of project name
							int i = label.indexOf(".");
							if(i != -1) {
								label = label.substring(i + 1);
							}
						}
						labelToValueMap.put(label, v);
					} else {
						labelToValueMap.put(computeFullyName((INamedElement)v), v);
					}
				} else {
					labelToValueMap.put(v.toString(), v);
				}
			}
		}
	}

	/** Returns the set of labels managed by this {@link ComboBoxLabelValueMapping}. */
	public Collection<String> getLabels() {
		if(labelToValueMap == null) {
			return stringLabels;
		}

		return labelToValueMap.keySet();
	}

	/** Translates a value into the corresponding label. */
	public String getLabelForValue(Object value) {
		if(labelToValueMap == null) {
			return value.toString();
		}

		for(String key : labelToValueMap.keySet()) {
			Object currValue = labelToValueMap.get(key);

			if(value == currValue) {
				return key;
			}
		}

		return "";
	}

	/** Translates a label into the corresponding value. */
	public Object getValueForLabel(String label) {
		if(labelToValueMap == null) {
			return label;
		}

		return labelToValueMap.get(label);
	}
}
